/******************************************************************************
*
* project name:   FAT-Engine
* file name:      fatd5.c
* initial date:   03/03/2002
* authors:        thomas.nussbaumer@gmx.net
*                 Kurt Hoffmann (LinkPlay stuff)
*                 Jeff Wilcox
*                 Scott Noveck
*
* description:    FAT-Engine - Demo 5 (LinkPlay Demo)
*
* $Id: fatd5.c,v 1.18 2002/05/08 15:35:14 tnussb Exp $
*
******************************************************************************/

#define USE_TI89
#define USE_TI92P
#define USE_V200

#include "../includes/fat.h"


#define  MAP_WIDTH        12
#define  MAP_HEIGHT       12

//-------------------------------------------------------------------
// defines the used map: 0 = no wall, 1 = texture1, 2 = texture2 etc.
// NOTE: the array tex64 which gets handed over to the engine as
//       part of the FATCONFIG structure has to as much textures as
//       you have specified in this map.
//       For example: if the highest number in the map is 3 then
//       tex64 has to hold 3 textures (64x64 pixels with 2 planes)
//-------------------------------------------------------------------
char map[MAP_WIDTH*MAP_HEIGHT] = {
     2, 1, 2, 2, 2, 2, 2, 2, 1, 1, 2, 1,
     2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
     2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2,
     2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2,
     2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
     1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
     2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2,
     1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
     2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
     1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
     1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2,
     1, 2, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1
};


//-----------------------------------------------------------------------------
// include the utility routines ...
//-----------------------------------------------------------------------------
#include "utils.c"

//-----------------------------------------------------------------------------
// include file containing the texture and genericdata indices into fatdtx
// (automatically generated by TexMaker)
//-----------------------------------------------------------------------------
#include "indices.h"

short display_offsetx = 0; // will be set depending on calctype (TI89 or TI92p)
short display_offsety = 0; // will be set depending on calctype (TI89 or TI92p)


//=============================================================================
// initializes GUI (darkgray background, tiny logo and counter)
//=============================================================================
void InitGUI(COUNTER *counter) {
    static unsigned long empty[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

    memset(GetPlane(LIGHT_PLANE),0,LCD_SIZE);
    memset(GetPlane(DARK_PLANE),0xff,LCD_SIZE);

    Sprite32(158-32+display_offsetx,display_offsety+2,16,empty,GetPlane(0),SPRT_AND);
    Sprite32(158-32+display_offsetx,display_offsety+2,16,FAT_GetTinyLogo(),GetPlane(0),SPRT_OR);
    Sprite32(158-32+display_offsetx,display_offsety+2,16,empty,GetPlane(1),SPRT_AND);
    Sprite32(158-32+display_offsetx,display_offsety+2,16,FAT_GetTinyLogo()+16,GetPlane(1),SPRT_OR);

    InitCounter(counter,158-6*5+display_offsetx,99-10+display_offsety,5,0);
    DrawCounter(counter);
}


//=============================================================================
// output frame count ever 10th frame
//=============================================================================
void frame_drawn(FATCONFIG* fc,COUNTER* counter) {
    short i;
    if (fc->frame_counter%10==0) {
        // we starting incrementing at 2nd position and not on first!
        for (i=counter->length-2;i>=0;i--) {

            counter->newcount[i] = counter->newcount[i]+1;
            if (counter->newcount[i] < 10) break;
            else                           counter->newcount[i]=0;
        }
        DrawCounter(counter);
    }
}


//#############################################################################
//#############################################################################
// Link play support functions by Kurt Hoffmann (START)
//#############################################################################
//#############################################################################
//-----------------------------------------------------------------------------
// a few network globals ...
//-----------------------------------------------------------------------------
#define LINK_CONNECTED 1
#define LINK_SERVER    2
#define LINK_SYNC      8
#define LINK_SEND      16
#define LINK_PING      50
#define LINK_PONG      60
#define LINK_RING      70
#define LINK_RONG      80
#define LINK_MSG       90
#define LINK_TIMELIM   5

short         cvars = LINK_SEND;
short         timeout = LINK_TIMELIM;
unsigned char rbuffer[10];

//=============================================================================
// send a ping to the other calc ...
//=============================================================================
void LINKONLYSendPing(unsigned char val) {
    OSWriteLinkBlock(&val,1);
}

//=============================================================================
// send location to the other calc ...
//=============================================================================
void LINKONLYSendData(const unsigned char* data, short num) {
    short count = 0;
    unsigned char sbuffer[num+2];
    sbuffer[0] = LINK_MSG;
    for (count = 0; count < num; count++) {
        sbuffer[count+1] = data[count];
    }
    sbuffer[count+1] = LINK_MSG;
    if (OSLinkTxQueueInquire() < 110) OSLinkReset();
    OSWriteLinkBlock(sbuffer, num+2);
}

//=============================================================================
// send location to the other calc ...
//=============================================================================
unsigned char* LinkConnection(const unsigned char* data, short num) {
    unsigned char* rval = NULL;
    memset(rbuffer, '\0', 10);

    if (!(cvars & LINK_CONNECTED)) {
        OSReadLinkBlock(rbuffer,1);
        if (rbuffer[0] == LINK_PING || rbuffer[0] == LINK_PONG) {
            timeout = LINK_TIMELIM;
            cvars |= LINK_CONNECTED;
            //-------------------------------------------------------------
            // if I got a ping, I am the server, send back a pong to client
            //-------------------------------------------------------------
            if (rbuffer[0] == LINK_PING) {
                LINKONLYSendPing(LINK_PONG);
                LINKONLYSendData(data, num);
                cvars |= LINK_SERVER;
            }
        }
        //-----------------------------------------------------------------
        // send out a ping once and only once at startup ...
        //-----------------------------------------------------------------
        if (!(cvars & LINK_CONNECTED) && (cvars & LINK_SEND)) {
            LINKONLYSendPing(LINK_PING);
            cvars ^= LINK_SEND;
        }
    }
    else if (cvars & LINK_SYNC) {
        //-----------------------------------------------------------------
        // if the link times out, try to sync
        //-----------------------------------------------------------------
        OSReadLinkBlock(rbuffer,1);
        if (rbuffer[0] == LINK_RING || rbuffer[0] == LINK_RONG) {
            timeout = LINK_TIMELIM;
            cvars ^= LINK_SYNC;
            if (rbuffer[0] == LINK_RING) {
                LINKONLYSendPing(LINK_RONG);
            }
            if (cvars & LINK_SERVER) {
                LINKONLYSendData(data, num);
            }
        }
        else {
            LINKONLYSendPing(LINK_RING);
        }
    }
    else {
        //----------------------------------------------------
        // we are connected, read opponent data ...
        //----------------------------------------------------
        OSReadLinkBlock(rbuffer, num+2);
        if (rbuffer[0] == LINK_MSG) {
            timeout = LINK_TIMELIM;
            if (rbuffer[num+1] == LINK_MSG) {
                rval = &rbuffer[1];
            }
            else {
              //-----------------------------------------------------------------
              // we have part of the message, just read the rest and send our
              // location and hope we get all of the message next time ...
              //-----------------------------------------------------------------
              rbuffer[0] = 0;
              do {
                  OSReadLinkBlock(rbuffer, 1);
                  if (rbuffer[0] == LINK_RING) {
                      cvars |= LINK_SYNC;
                      rbuffer[0] = LINK_MSG;
                  }
              } while(rbuffer[0] != LINK_MSG);
            }
            LINKONLYSendData(data, num);
        }
        else if (timeout-- <= 0) {
            cvars |= LINK_SYNC;
        }
    }
    return rval;
}

//=============================================================================
// is the link up ...
//=============================================================================
//BOOL LinkUp() {
//    return ((cvars & LINK_CONNECTED) != 0 && (cvars & LINK_SYNC) == 0);
//}

//=============================================================================
// calc num (master/slave = first/second) ...
//=============================================================================
//BOOL LinkMaster() {
//    return ((cvars & LINK_SERVER) != 0);
//}

//#############################################################################
//#############################################################################
// Link play support functions by Kurt Hoffmann (END)
//#############################################################################
//#############################################################################



//=============================================================================
// as usual ... our main function
//=============================================================================
void _main(void) {
    PADSTATES       key    = 0;
    unsigned char*  bg_img = NULL;
    short           i,j;
    unsigned short* dest;
    TEXCONFIG       textures[2];
    TEXCONFIG       sprite_tex[8];  // eight sprite textures (one for each direction)
    FATSPRITE       sprite;         // one FAT sprite
    LCD_BUFFER      lcd;
    HANDLE          texfile = 0;
    unsigned char*  src     = NULL;
    unsigned char   sendbuffer[6];  // used to send position to another calc
    unsigned char   *recv;          // received data from other calc
    short           result;
    COUNTER         frames_display;

    FATCONFIG fc = {
        MAP_WIDTH, // width of map
        MAP_HEIGHT,// height of map
        map,       // pointer to map data
        textures,  // pointer to textures array
        1,         // number of sprites
        &sprite,   // the sprite array
        96+64*3,   // start pos_x
        96+64*5,   // start pos_y
        ANGLE_EAST,// orientation
        NULL,      // destination plane0
        NULL,      // destination plane1
        NULL,      // background image
        0          // frame counter
    };

    //-------------------------------------------------------------------------
    // before we try to do anything else we have to initialize the engine
    // its only valid to proceed if FAT_LOADOKAY gets returned
    //-------------------------------------------------------------------------
    if ((result = FAT_LoadEngine()) != FAT_LOADOKAY) {
        switch(result) {
            case FAT_NOTGHOSTSPACE: ST_helpMsg("ERROR: client no exepacked!");break;
            case FAT_NOTFOUND:      ST_helpMsg("ERROR: no matching engine found");break;
            case FAT_LOCKFAILED:    ST_helpMsg("ERROR: locking engine failed");break;
            case FAT_OUTOFMEM:      ST_helpMsg("ERROR: out of memory");break;
            default:                ST_helpMsg("ERROR: unknown errorcode");break;
        }
        return;
    }

    //-------------------------------------------------------------------------
    // try to load the used textures from fatdtx ...
    //-------------------------------------------------------------------------
    if ((texfile = FAT_GetHandleOfFile("fatdtx"))) {
        if ((src = FAT_LockHandleOfFile(texfile))) {
            // load house wall textures
            if (!FAT_LoadTextures(textures,src+2,TEXIDX_HOUSE1,2)) {
                src = 0; // just to indicate something went wrong
            }
            // load texture for doom guy
            if (src) {
                if (!FAT_LoadTextures(sprite_tex,src+2,TEXIDX_SOLDIER1,8)) {
                    src = 0; // just to indicate something went wrong
                }
            }
        }
    }

    //-------------------------------------------------------------------------
    // check if loading failed ...
    //-------------------------------------------------------------------------
    if (!texfile || !src) {
        FAT_UnloadEngine();
        if (texfile) FAT_UnlockHandleOfFile(texfile);
        ST_helpMsg("ERROR: cannot textures from fatdtx");
        return;
    }

    LCD_save(lcd);

    //-------------------------------------------------------------------------
    // initialize sprite structure and setup its fields
    //-------------------------------------------------------------------------
    FAT_InitSprite(&sprite);
    sprite.drawmode    = SPRITEMODE_SIZE64;
    sprite.nr_textures = 8;
    sprite.textures    = sprite_tex;
    sprite.orientation = 0;
    sprite.xpos        = 96+64*5;
    sprite.ypos        = 96+64*5;


    //-------------------------------------------------------------------------
    // allocate space for a background image and fill the lower part of the
    // light plane with 0xff to simulate a floor ...
    //-------------------------------------------------------------------------
    if (!(bg_img = (unsigned char*)malloc(1152*2))) {
        FAT_UnloadEngine();
        FAT_UnlockHandleOfFile(texfile);
        ST_helpMsg("ERROR: out of mem");
        return;
    }
    memset(bg_img,0x00,1152*2);

    dest = (unsigned short*)bg_img;
    for (i=0;i<6;i++) {
        for (j=0;j<48;j++) *dest++ = 0x0000;
        for (j=0;j<48;j++) *dest++ = 0xffff;
    }

    fc.background = (unsigned short*)bg_img;

    InstallDummyHandlerForINTs();

    //-------------------------------------------------------------------------
    // initialize grayscales, clear both planes and set the destination plane
    // addresses for the FAT-Engine
    //-------------------------------------------------------------------------
    if (!GrayOn()) {
        free(bg_img);
        RestoreHandlerForINTs();
        FAT_UnloadEngine();
        FAT_UnlockHandleOfFile(texfile);
        ST_helpMsg("ERROR: out of mem");
        return;
    }

    //-------------------------------------------------------------------------
    // display the FAT-Engine's splashscreen
    //-------------------------------------------------------------------------
    FAT_DisplayLogo(100);

    fc.dest_plane0 = GetPlane(0)+30*2;  // center vertically
    fc.dest_plane1 = GetPlane(1)+30*2;  // center vertically

    //-------------------------------------------------------------------------
    // setup calculator type specials
    //-------------------------------------------------------------------------
    if (TI89) {
        display_offsetx = 0;
        display_offsety = 0;
    }
    else {
        // TI92p or V200
        display_offsetx  = 40;
        display_offsety  = 14;
        fc.dest_plane0  += (14*30+4)/2;
        fc.dest_plane1  += (14*30+4)/2;
    }

    //-------------------------------------------------------------------------
    // initialize the GUI
    //-------------------------------------------------------------------------
    InitGUI(&frames_display);

    //-------------------------------------------------------------------------
    // the main rendering loop ...
    //-------------------------------------------------------------------------
    do {
        FAT_Render(&fc);   // render image

        frame_drawn(&fc,&frames_display);  // handles frame counter drawing


        //#####################################################################
        // begin link CODE of Kurt Hoffmann
        //#####################################################################
        sendbuffer[0] = fc.cam_xpos - ((fc.cam_xpos>>8)<<8);
        sendbuffer[1] = fc.cam_xpos>>8;
        sendbuffer[2] = fc.cam_ypos - ((fc.cam_ypos>>8)<<8);
        sendbuffer[3] = fc.cam_ypos>>8;
        sendbuffer[4] = fc.cam_orientation - ((fc.cam_orientation>>8)<<8);
        sendbuffer[5] = fc.cam_orientation>>8;
        if ((recv = LinkConnection(sendbuffer, 6))) {
            sprite.xpos = recv[0] + (recv[1]<<8);
            sprite.ypos = recv[2] + (recv[3]<<8);
            sprite.orientation = (recv[4] + (recv[5]<<8)) % ANGLE_360;
        }
        //#####################################################################
        // end link CODE of Kurt Hoffmann
        //#####################################################################

        if ((key = FAT_GetPadState())) {
           if (key & PADSTATE_ESC)    break;
           if (key & PADSTATE_UPDOWN) PlayerMove(&fc,key & PADSTATE_UP,0);

           if (key & PADSTATE_LEFTRIGHT) {
               // STRAFE KEY: SHIFT (TI-89) / F1 (TI-92p)
               if (key & PADSTATE_KA) {
                   PlayerMove(&fc,key&PADSTATE_RIGHT,1);
               }
               else {
                   PlayerTurn(&fc,key&PADSTATE_RIGHT,ANGLE_015/2);
               }
           }
        }
    }
    while(1);

    //-------------------------------------------------------------------------
    // some clean stuff ...
    //-------------------------------------------------------------------------
    FAT_AllKeysReleased();
    GrayOff();
    free(bg_img);
    LCD_restore(lcd);
    ST_helpMsg(FAT_GetPoweredByString());
    FAT_UnlockHandleOfFile(texfile);  // unlock texture file
    FAT_UnloadEngine();
    RestoreHandlerForINTs();
    OSqclear(kbd_queue());
}

//#############################################################################
// Revision History
//#############################################################################
//
// $Log: fatd5.c,v $
// Revision 1.18  2002/05/08 15:35:14  tnussb
// (1) display of FAT splashscreen (FAT_DisplayLogo) added
// (2) GUI of demos completely revised
//
// Revision 1.17  2002/05/07 15:22:13  tnussb
// stupid double call of FAT_LoadEngine() fixed
//
// Revision 1.16  2002/04/30 17:41:44  tnussb
// using now defines from automatically generated headerfile indices.h for indices into
// our texture file
//
// Revision 1.15  2002/04/26 13:10:49  tnussb
// if FAT_LoadEngine() fails different strings will be displayed yet
//
// Revision 1.14  2002/04/25 19:32:03  tnussb
// generic commit for engine version v1.05
//
// Revision 1.13  2002/04/24 17:41:54  tnussb
// (1) using now FAT_GetPadState() to query key presses
// (2) movement evaluation block rewritten (handles now strafing, too)
//
// Revision 1.12  2002/04/18 17:08:17  tnussb
// linkplay code of Kurt Hoffmann added
//
// Revision 1.11  2002/04/18 12:14:47  tnussb
// using now fatdtx texture file instead of embedded textures
//
// Revision 1.10  2002/04/16 11:36:37  tnussb
// All calls to GrayMode() replaced by GrayOn/GrayOff calls
//
// Revision 1.9  2002/04/15 10:00:12  tnussb
// using now STRIP_NORMAL instead of 0 for mirrored field structure TEXSTRIP
//
// Revision 1.8  2002/04/08 20:18:16  tnussb
// using now signed char (instead of unsigned char) for map
//
// Revision 1.7  2002/03/26 11:56:24  tnussb
// all warnings reported by -Wall fixed
//
// Revision 1.6  2002/03/12 14:36:04  tnussb
// uses now FAT_InitSprite() and setup drawmode of sprite correctly
//
// Revision 1.5  2002/03/11 09:17:42  tnussb
// USE_V200 added
//
// Revision 1.4  2002/03/08 13:37:04  tnussb
// multidirectional sprite support is now part of the engine (removed code from
// here)
//
// Revision 1.3  2002/03/07 11:16:24  tnussb
// restores now previously set font at the end
//
// Revision 1.2  2002/03/06 12:31:01  tnussb
// (1) uses now Scott's fast direction evaluation
// (2) OutputTitle completely rewritten (uses now new exported "functions" of FAT-Engine)
// (3) InstallDummyHandleForINT1(), RestoreHandlerForINT1() renamed, because AUTO_INT5 will be redirected now, too
// (4) don't use SAVE_SCREEN anymore, but use own screenbackup to enable writting to the statusline
//
// Revision 1.1  2002/03/01 17:26:26  tnussb
// initial check-in
//
//
//
