/******************************************************************************
*
* project name:   FAT-Engine
* file name:      fatd7.c
* initial date:   03/23/2002
* authors:        Scott Noveck
*                 thomas.nussbaumer@gmx.net
*
* description:    FAT-Engine - Demo 7 (rotating sprite)
*
* UP/DOWN     ... zoom in/out
* LEFT/RIGHT  ... rotate camera around sprite
*
******************************************************************************/

#define USE_TI89
#define USE_TI92P
#define USE_V200

#include "../includes/fat.h"


#define  MAP_WIDTH        8
#define  MAP_HEIGHT       12

//-------------------------------------------------------------------
// defines the used map: 0 = no wall, 1 = texture1, 2 = texture2 etc.
// NOTE: the array tex64 which gets handed over to the engine as
//       part of the FATCONFIG structure has to as much textures as
//       you have specified in this map.
//       For example: if the highest number in the map is 3 then
//       tex64 has to hold 3 textures (64x64 pixels with 2 planes)
//-------------------------------------------------------------------

// use an empty map for this demo
unsigned char map[MAP_WIDTH*MAP_HEIGHT] = {
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0,
};


//-----------------------------------------------------------------------------
// include the utility routines ...
//-----------------------------------------------------------------------------
#include "utils.c"

//-----------------------------------------------------------------------------
// include file containing the texture and genericdata indices into fatdtx
// (automatically generated by TexMaker)
//-----------------------------------------------------------------------------
#include "indices.h"


short display_offsetx = 0; // will be set depending on calctype (TI89 or TI92p)
short display_offsety = 0; // will be set depending on calctype (TI89 or TI92p)


//=============================================================================
// initializes GUI (darkgray background, tiny logo and counter)
//=============================================================================
void InitGUI(COUNTER *counter) {
    static unsigned long empty[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

    memset(GetPlane(LIGHT_PLANE),0,LCD_SIZE);
    memset(GetPlane(DARK_PLANE),0xff,LCD_SIZE);

    Sprite32(158-32+display_offsetx,display_offsety+2,16,empty,GetPlane(0),SPRT_AND);
    Sprite32(158-32+display_offsetx,display_offsety+2,16,FAT_GetTinyLogo(),GetPlane(0),SPRT_OR);
    Sprite32(158-32+display_offsetx,display_offsety+2,16,empty,GetPlane(1),SPRT_AND);
    Sprite32(158-32+display_offsetx,display_offsety+2,16,FAT_GetTinyLogo()+16,GetPlane(1),SPRT_OR);

    InitCounter(counter,158-6*5+display_offsetx,99-10+display_offsety,5,0);
    DrawCounter(counter);
}


//=============================================================================
// output frame count ever 10th frame
//=============================================================================
void frame_drawn(FATCONFIG* fc,COUNTER* counter) {
    short i;
    if (fc->frame_counter%10==0) {
        // we starting incrementing at 2nd position and not on first!
        for (i=counter->length-2;i>=0;i--) {

            counter->newcount[i] = counter->newcount[i]+1;
            if (counter->newcount[i] < 10) break;
            else                           counter->newcount[i]=0;
        }
        DrawCounter(counter);
    }
}


//=============================================================================
// as usual ... our main function
//=============================================================================
void _main(void) {
    PADSTATES      key = 0;
    TEXCONFIG      sprite_tex[8];            // eight textures for the sprite
    FATSPRITE      sprite;                   // the sprite structure itself
    LCD_BUFFER     lcd;
    short          dist_from_sprite;
    short          angle_from_sprite;
    short          angle_delta;
    HANDLE         texfile = 0;
    unsigned char* src = NULL;
    short          result;
    COUNTER        frames_display;

    FATCONFIG fc = {
        MAP_WIDTH,  // width of map
        MAP_HEIGHT, // height of map
        map,        // pointer to map data
        NULL,       // no textures for walls
        1,          // number of sprites
        &sprite,    // the sprite array
        96,         // start pos_x
        96,         // start pos_y
        ANGLE_EAST, // orientation
        NULL,       // destination plane0
        NULL,       // destination plane1
        NULL,       // background image
        0           // frame counter
    };

    //-------------------------------------------------------------------------
    // before we try to do anything else we have to initialize the engine
    // its only valid to proceed if FAT_LOADOKAY gets returned
    //-------------------------------------------------------------------------
    if ((result = FAT_LoadEngine()) != FAT_LOADOKAY) {
        switch(result) {
            case FAT_NOTGHOSTSPACE: ST_helpMsg("ERROR: client no exepacked!");break;
            case FAT_NOTFOUND:      ST_helpMsg("ERROR: no matching engine found");break;
            case FAT_LOCKFAILED:    ST_helpMsg("ERROR: locking engine failed");break;
            case FAT_OUTOFMEM:      ST_helpMsg("ERROR: out of memory");break;
            default:                ST_helpMsg("ERROR: unknown errorcode");break;
        }
        return;
    }

    //-------------------------------------------------------------------------
    // try to load the textures of the doom soldier from fatdtx ...
    //-------------------------------------------------------------------------
    if ((texfile = FAT_GetHandleOfFile("fatdtx"))) {
        if ((src = FAT_LockHandleOfFile(texfile))) {
            if (!FAT_LoadTextures(sprite_tex,src+2,TEXIDX_SOLDIER1,8)) {
                src = 0; // just to indicate something went wrong
            }
        }
    }

    //-------------------------------------------------------------------------
    // check if loading failed ...
    //-------------------------------------------------------------------------
    if (!texfile || !src) {
        FAT_UnloadEngine();
        if (texfile) FAT_UnlockHandleOfFile(texfile);
        ST_helpMsg("ERROR: cannot textures from fatdtx");
        return;
    }

    LCD_save(lcd);

    //-------------------------------------------------------------------------
    // initialize sprite structure and setup its fields
    //-------------------------------------------------------------------------
    FAT_InitSprite(&sprite);
    sprite.drawmode    = SPRITEMODE_SIZE64;
    sprite.nr_textures = 8;
    sprite.textures    = sprite_tex;
    sprite.orientation = 0;
    sprite.xpos        = (MAP_WIDTH*64)/2;     // center horizontally
    sprite.ypos        = (MAP_HEIGHT*64)/2;    // center vertically

    InstallDummyHandlerForINTs();

    //-------------------------------------------------------------------------
    // initialize grayscales, clear both planes and set the destination plane
    // addresses for the FAT-Engine
    //-------------------------------------------------------------------------
    if (!GrayOn()) {
        FAT_UnloadEngine();               // unload engine again
        FAT_UnlockHandleOfFile(texfile);  // unlock texture file
        RestoreHandlerForINTs();
        ST_helpMsg("ERROR: out of mem");
        return;
    }

    //-------------------------------------------------------------------------
    // display the FAT-Engine's splashscreen
    //-------------------------------------------------------------------------
    FAT_DisplayLogo(100);

    fc.dest_plane0 = GetPlane(0)+30*2;
    fc.dest_plane1 = GetPlane(1)+30*2;

    //-------------------------------------------------------------------------
    // setup calculator type specials
    //-------------------------------------------------------------------------
    if (TI89) {
        display_offsetx = 0;
        display_offsety = 0;
    }
    else {
        // TI92p or V200
        display_offsetx  = 40;
        display_offsety  = 14;
        fc.dest_plane0  += (14*30+4)/2;
        fc.dest_plane1  += (14*30+4)/2;
    }

    //-------------------------------------------------------------------------
    // initialize the GUI
    //-------------------------------------------------------------------------
    InitGUI(&frames_display);

    dist_from_sprite  = 15;            // in number of "steps"
    angle_from_sprite = ANGLE_000;
    angle_delta       = 0;

#define ANGLE_DELTA_DELTA 4
    //-------------------------------------------------------------------------
    // the main rendering loop ...
    //-------------------------------------------------------------------------
    do {
        // set camera position
        fc.cam_xpos = sprite.xpos + ((dist_from_sprite*((long)FAT_Cos16384(angle_from_sprite))+(2<<10)) >> 11);
        fc.cam_ypos = sprite.ypos + ((dist_from_sprite*((long)FAT_Sin16384(angle_from_sprite))+(2<<10)) >> 11);

        fc.cam_orientation = angle_from_sprite-ANGLE_180;
        if (fc.cam_orientation < 0)  fc.cam_orientation += ANGLE_360;

        FAT_Render(&fc);                  // render image
        frame_drawn(&fc,&frames_display); // update frame counter

        if ((key = FAT_GetPadState())) {
            if (key&PADSTATE_ESC)    break;
            if ((key&PADSTATE_UP)    && (dist_from_sprite> 4))                       dist_from_sprite--;
            if ((key&PADSTATE_DOWN)  && (dist_from_sprite<25))                       dist_from_sprite++;
            if ((key&PADSTATE_LEFT)  && (angle_delta>=-ANGLE_045+ANGLE_DELTA_DELTA)) angle_delta-=ANGLE_DELTA_DELTA;
            if ((key&PADSTATE_RIGHT) && (angle_delta<= ANGLE_045-ANGLE_DELTA_DELTA)) angle_delta+=ANGLE_DELTA_DELTA;
        }

        // adjust distance and camera orientation -- in key detection above
        angle_from_sprite += angle_delta;
        if (angle_from_sprite < ANGLE_000)       angle_from_sprite += ANGLE_360;
        else if (angle_from_sprite >= ANGLE_360) angle_from_sprite -= ANGLE_360;
    }
    while(1);

    //-------------------------------------------------------------------------
    // some clean stuff ...
    //-------------------------------------------------------------------------
    FAT_AllKeysReleased();
    GrayOff();
    LCD_restore(lcd);
    ST_helpMsg(FAT_GetPoweredByString());
    FAT_UnlockHandleOfFile(texfile);  // unlock texture file
    FAT_UnloadEngine();
    RestoreHandlerForINTs();
    OSqclear(kbd_queue());
}

//#############################################################################
// Revision History
//#############################################################################
//
// $Log: fatd7.c,v $
// Revision 1.13  2002/05/08 15:35:14  tnussb
// (1) display of FAT splashscreen (FAT_DisplayLogo) added
// (2) GUI of demos completely revised
//
// Revision 1.12  2002/05/07 15:22:13  tnussb
// stupid double call of FAT_LoadEngine() fixed
//
// Revision 1.11  2002/04/30 17:41:44  tnussb
// using now defines from automatically generated headerfile indices.h for indices into
// our texture file
//
// Revision 1.10  2002/04/26 13:10:49  tnussb
// if FAT_LoadEngine() fails different strings will be displayed yet
//
// Revision 1.9  2002/04/25 19:34:26  tnussb
// generic commit for engine version v1.05
//
// Revision 1.8  2002/04/24 17:42:51  tnussb
// (1) using now FAT_GetPadState() to query key presses
//
// Revision 1.7  2002/04/18 12:14:47  tnussb
// using now fatdtx texture file instead of embedded textures
//
// Revision 1.6  2002/04/17 16:47:03  tnussb
// debug output removed
//
// Revision 1.5  2002/04/17 10:04:22  tnussb
// using now macros FAT_Cos16384/FAT_Sin16384
//
// Revision 1.4  2002/04/16 11:36:38  tnussb
// All calls to GrayMode() replaced by GrayOn/GrayOff calls
//
// Revision 1.3  2002/04/15 10:00:38  tnussb
// (1) using now STRIP_NORMAL instead of 0 for mirrored field structure TEXSTRIP
// (2) displays now field angle of FATSPRITE structure
//
// Revision 1.1  2002/03/26 11:55:22  tnussb
// initial version
//
//
