/******************************************************************************
*
* project name:   FAT-Engine
* file name:      fatd8.c
* initial date:   16/04/2002
* authors:        Scott Noveck (third person view implementation)
*                 thomas.nussbaumer@gmx.net
*
* description:    FAT-Engine - Demo 8 (Third-Person View)
*
* $Id: fatd8.c,v 1.8 2002/05/08 15:35:14 tnussb Exp $
*
******************************************************************************/

#define USE_TI89
#define USE_TI92P
#define USE_V200

#include "../includes/fat.h"


// this flag forces the camera to remain behind the sprite; otherwise,
//it moves more freely

//#define FORCE_CAMERA_BEHIND

#define ANGLE_135    (ANGLE_090+ANGLE_045)
#define ANGLE_225    (ANGLE_180+ANGLE_045)
#define ANGLE_315    (ANGLE_270+ANGLE_045)

#define MAX_CAM_DIST    (16*8)        // 8 is one step, 64 is one tile
#define MIN_CAM_DIST    ( 8*8)

#define MIN_WALL_DIST     16

#define MAP_WIDTH         12
#define MAP_HEIGHT        12

//-------------------------------------------------------------------
// defines the used map: 0 = no wall, 1 = texture1, 2 = texture2 etc.
// NOTE: the array tex64 which gets handed over to the engine as
//       part of the FATCONFIG structure has to as much textures as
//       you have specified in this map.
//       For example: if the highest number in the map is 3 then
//       tex64 has to hold 3 textures (64x64 pixels with 2 planes)
//-------------------------------------------------------------------
char map[MAP_WIDTH*MAP_HEIGHT] = {
     2, 1, 2, 2, 1, 1, 2, 1, 2, 2, 1, 1,
     2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
     2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2,
     2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2,
     2, 0, 0, 0, 0, 0, 0, 2, 1, 0, 0, 1,
     1, 0, 1, 0, 0, 0, 0, 1, 2, 0, 0, 1,
     2, 0, 2, 0, 0, 0, 0, 2, 1, 0, 0, 2,
     1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1,
     2, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 1,
     1, 0, 2, 1, 1, 1, 0, 0, 0, 0, 0, 1,
     1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2,
     1, 2, 1, 1, 2, 2, 1, 1, 2, 1, 2, 2,
};


//-----------------------------------------------------------------------------
// include the utility routines ...
//-----------------------------------------------------------------------------
#include "utils.c"

//-----------------------------------------------------------------------------
// include file containing the texture and genericdata indices into fatdtx
// (automatically generated by TexMaker)
//-----------------------------------------------------------------------------
#include "indices.h"


short display_offsetx = 0; // will be set depending on calctype (TI89 or TI92p)
short display_offsety = 0; // will be set depending on calctype (TI89 or TI92p)


//=============================================================================
// initializes GUI (darkgray background, tiny logo and counter)
//=============================================================================
void InitGUI(COUNTER *counter) {
    static unsigned long empty[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

    memset(GetPlane(LIGHT_PLANE),0,LCD_SIZE);
    memset(GetPlane(DARK_PLANE),0xff,LCD_SIZE);

    Sprite32(158-32+display_offsetx,display_offsety+2,16,empty,GetPlane(0),SPRT_AND);
    Sprite32(158-32+display_offsetx,display_offsety+2,16,FAT_GetTinyLogo(),GetPlane(0),SPRT_OR);
    Sprite32(158-32+display_offsetx,display_offsety+2,16,empty,GetPlane(1),SPRT_AND);
    Sprite32(158-32+display_offsetx,display_offsety+2,16,FAT_GetTinyLogo()+16,GetPlane(1),SPRT_OR);

    InitCounter(counter,158-6*5+display_offsetx,99-10+display_offsety,5,0);
    DrawCounter(counter);
}


//=============================================================================
// output frame count ever 10th frame
//=============================================================================
void frame_drawn(FATCONFIG* fc,COUNTER* counter) {
    short i;
    if (fc->frame_counter%10==0) {
        // we starting incrementing at 2nd position and not on first!
        for (i=counter->length-2;i>=0;i--) {

            counter->newcount[i] = counter->newcount[i]+1;
            if (counter->newcount[i] < 10) break;
            else                           counter->newcount[i]=0;
        }
        DrawCounter(counter);
    }
}


//=============================================================================
// checks if the given positions is not too near to a solid square
//=============================================================================
short ValidMove(FATCONFIG* fc, short x, short y) {
    // if we would come too near to a solid square...
    if ((*(map+((y+MIN_WALL_DIST)>>6)*fc->map_width+((x+MIN_WALL_DIST)>>6))>0) ||
        (*(map+((y-MIN_WALL_DIST)>>6)*fc->map_width+((x-MIN_WALL_DIST)>>6))>0) ||
        (*(map+((y+MIN_WALL_DIST)>>6)*fc->map_width+((x-MIN_WALL_DIST)>>6))>0) ||
        (*(map+((y-MIN_WALL_DIST)>>6)*fc->map_width+((x+MIN_WALL_DIST)>>6))>0))
    {
       return 0;        // false
    }
    return 1;           // true
}


//=============================================================================
// similar check like above, but for the camera position ...
//=============================================================================
inline short ValidCamMove(FATCONFIG* fc, short x, short y) {
    if (*(map+(y>>6)*fc->map_width+(x>>6))>0)  return 0;        // false
    else                                       return 1;        // true
}

//=============================================================================
// When all attempts at repositioning the camera fail, resort to this
// last-ditch routine could still have problems if view of sprite is partially
// blocked after moving
//=============================================================================
void ResetCam(FATCONFIG* fc, FATSPRITE* sprite, short movingbackwards) {
    short newxpos, newypos, count = 0;

    // first try to zoom in
    // angle from cam to sprite
    short dx = sprite->xpos - fc->cam_xpos;
    short dy = sprite->ypos - fc->cam_ypos;
    short angle = FAT_GetArcTanYX(dy, dx);
    short xstep = (4*((long)FAT_Cos16384(angle)))<<14;
    short ystep = (4*((long)FAT_Cos16384(angle)))<<14;
    short distance = dx*dx+dy*dy;
    if (((long)(FAT_GetDepthBuffer()->distance[SCREEN_WIDTH_HALF]) *
         (long)(FAT_GetDepthBuffer()->distance[SCREEN_WIDTH_HALF])) > distance)
    {
        for (distance=FAT_FastSqrt(distance)-4; distance>=MIN_CAM_DIST; distance-=4) {
            fc->cam_xpos += xstep;
            fc->cam_ypos += ystep;
            if (ValidCamMove(fc, fc->cam_xpos, fc->cam_ypos)) {
                fc->cam_orientation = angle;
                return;
            }
        }
    }

    // if zooming failed to find a valid move, zoom all the way in and
    // pan looking for valid move
    #ifdef FORCE_CAMERA_BEHIND
        angle = sprite->orientation-ANGLE_225;
    #else
        angle = sprite->orientation;
        if (!movingbackwards)  angle-=ANGLE_180;
    #endif
    if (angle < ANGLE_000)  angle += ANGLE_360;

    do {
        newxpos = sprite->xpos + ((MIN_CAM_DIST*(long)FAT_Cos16384(angle)) >> 14);
        newypos = sprite->ypos + ((MIN_CAM_DIST*(long)FAT_Sin16384(angle)) >> 14);

        if (ValidCamMove(fc, newxpos, newypos)) {
            fc->cam_xpos = newxpos;
            fc->cam_ypos = newypos;
            fc->cam_orientation = FAT_GetArcTanYX(sprite->ypos-fc->cam_ypos, sprite->xpos-fc->cam_xpos);
            return;
        }

        ++count;
        angle += ANGLE_045/2;
        if (angle >= ANGLE_360)  angle -= ANGLE_360;
    } while (count < 16);

    // Complete camera positioning failure.  This is bad.  Just place it somewhere!
    angle = sprite->orientation-ANGLE_180;
    if (angle < ANGLE_000)  angle += ANGLE_360;
    fc->cam_xpos = sprite->xpos + ((MIN_CAM_DIST*(long)FAT_Cos16384(angle)) >> 14);
    fc->cam_ypos = sprite->ypos + ((MIN_CAM_DIST*(long)FAT_Sin16384(angle)) >> 14);
}


//=============================================================================
// the following function forces the camera to be behind the sprite
//=============================================================================
#ifdef FORCE_CAMERA_BEHIND
void ForceCamBehind(FATCONFIG* fc, FATSPRITE* sprite) {
// Enable this code to force the camera to remain _behind_ the sprite
    short dx = sprite->xpos-fc->cam_xpos;
    short dy = sprite->ypos-fc->cam_ypos;
    short distance = dx*dx + dy*dy;
    // this calculates angle from _sprite_ to _cam_, not the other way
    short angle = FAT_GetArcTanYX(fc->cam_ypos-sprite->ypos, fc->cam_xpos-sprite->xpos) - sprite->orientation;
    if (angle < ANGLE_000)  angle += ANGLE_360;
    distance = FAT_FastSqrt(distance);

    if (angle < ANGLE_135) {
        fc->cam_xpos = sprite->xpos + ((distance*((long)FAT_Cos16384(ANGLE_135+sprite->orientation))) >> 14);
        fc->cam_ypos = sprite->ypos + ((distance*((long)FAT_Sin16384(ANGLE_135+sprite->orientation))) >> 14);
    }
    else if (angle > ANGLE_225) {
        fc->cam_xpos = sprite->xpos + ((distance*((long)FAT_Cos16384(ANGLE_225+sprite->orientation))) >> 14);
        fc->cam_ypos = sprite->ypos + ((distance*((long)FAT_Sin16384(ANGLE_225+sprite->orientation))) >> 14);
    }

    if (!ValidCamMove(fc, fc->cam_xpos, fc->cam_ypos))  ResetCam(fc, sprite, 0);
    else  fc->cam_orientation = FAT_GetArcTanYX(sprite->ypos-fc->cam_ypos, sprite->xpos-fc->cam_xpos);
}
#endif // FORCE_CAMERA_BEHIND


//=============================================================================
// treatment of camera rotation
//=============================================================================
void RotateCam(FATCONFIG* fc, FATSPRITE* sprite, short amount, short distancesquared) {
    short newxpos, newypos;
    short distance = FAT_FastSqrt(distancesquared);
    short angle = sprite->angle - ANGLE_180;
    if (angle < ANGLE_000)  angle += ANGLE_360;
    angle += amount;
    if (angle < ANGLE_000)        angle += ANGLE_360;
    else if (angle >= ANGLE_360)  angle -= ANGLE_360;

    newxpos = sprite->xpos + (distance*((long)FAT_Cos16384(angle)) >> 14);
    newypos = sprite->ypos + (distance*((long)FAT_Sin16384(angle)) >> 14);

    // camera rotation shouldn't fail; if only x or y component is valid, still move along that
    if (!ValidCamMove(fc, newxpos, newypos)) {  // if new position is invalid ...
        // ...but old position is OK, we're against a wall to east or west
        if (ValidCamMove(fc, fc->cam_xpos, newypos)) {
            // still allow movement along y = parallel to wall
            fc->cam_ypos = newypos;
        }
        // ...but (new x, old y) is OK, we're against a wall to north or south
        if (ValidCamMove(fc, newxpos, fc->cam_ypos)) {
            // still allow movement along x = parallel to wall
            fc->cam_xpos = newxpos;
        }
        // not likely to get here, but possible - do nothing
        else  {ResetCam(fc, sprite, 0); return;}
    }
    else {
        fc->cam_xpos = newxpos;
        fc->cam_ypos = newypos;
    }

    fc->cam_orientation = FAT_GetArcTanYX(sprite->ypos-fc->cam_ypos, sprite->xpos-fc->cam_xpos);
}


//=============================================================================
// turn the sprite
//=============================================================================
void SpriteTurn(FATCONFIG* fc, FATSPRITE* sprite,short  right) {
    fc = fc; // just to make the compiler happy ...
    sprite->orientation += (right) ? ANGLE_015 : -ANGLE_015;
    if (sprite->orientation >= ANGLE_360) sprite->orientation -= ANGLE_360;
    if (sprite->orientation < 0)          sprite->orientation += ANGLE_360;

#ifdef FORCE_CAMERA_BEHIND
    ForceCamBehind(fc, sprite);
#endif
}

//=============================================================================
// move player forward or backward depending on player's angle ...
// post-movement camera adjustment
//=============================================================================
void SpriteMove(FATCONFIG* fc, FATSPRITE* sprite,short  forward) {
    short stepx   = FAT_Cos16384(sprite->orientation) >> 11;
    short stepy   = FAT_Sin16384(sprite->orientation) >> 11;
    short newxpos = sprite->xpos + ((forward)? stepx: -stepx);
    short newypos = sprite->ypos + ((forward)? stepy: -stepy);
    short newxmod = newxpos >> 6;
    short newymod = newypos >> 6;
    short dx, dy;
    unsigned short distance;

    //-------------------------------------------------------------------------
    // check if we are still within the map
    //-------------------------------------------------------------------------
    if (newxmod > fc->map_width-1  ||
        newymod > fc->map_height-1 ||
        newxmod < 0 || newymod < 0)
    {
       return;
    }

    if (!ValidMove(fc, newxpos, newypos)) {  // if new position is invalid ...
        // ...but old position is OK, we're against a wall to east or west
        if (ValidMove(fc, sprite->xpos, newypos)) {
            // don't change angle if movement is perpendicular to the wall
            if (stepy != 0) {
                // if sign(stepx)==sign(stepy), rotate clockwise = turn right
                if ((stepx>0)==(stepy>0)) SpriteTurn(fc,sprite,1);
                else                      SpriteTurn(fc,sprite,0);
                // still allow movement along y = parallel to wall
                sprite->ypos = newypos;
            }
        }
        // ...but (new x, old y) is OK, we're against a wall to north or south
        if (ValidMove(fc, newxpos, sprite->ypos)) {
            // don't change angle if movement is perpendicular to the wall
            if (stepx != 0) {
               // if sign(stepx)==sign(stepy), rotate counterclockwise = turn left
               if ((stepx>0)==(stepy>0)) SpriteTurn(fc,sprite,0);
               else                      SpriteTurn(fc,sprite,1);
               // still allow movement along x = parallel to wall
               sprite->xpos = newxpos;
            }
        }
        // ...otherwise we're in a corner, so do nothing
        // else return;
    }
    else {
        sprite->xpos = newxpos;
        sprite->ypos = newypos;
    }

    //-------------------------------------------------------------------------
    // now we need to reposition the camera; this code will get ugly =)
    //-------------------------------------------------------------------------

    dx = sprite->xpos-fc->cam_xpos;
    dy = sprite->ypos-fc->cam_ypos;
    distance = dx*dx + dy*dy;

// (1) If camera is too close to objects, zoom out (with quick panning)
    while (distance < (MIN_CAM_DIST*MIN_CAM_DIST)) {
        fc->cam_xpos -= (8*(long)FAT_Cos16384(fc->cam_orientation) >> 14);
        fc->cam_ypos -= (8*(long)FAT_Sin16384(fc->cam_orientation) >> 14);

        dx = sprite->xpos-fc->cam_xpos;
        dy = sprite->ypos-fc->cam_ypos;
        distance = dx*dx + dy*dy;
    }

// (2) If camera is too far, zoom in (with gradual panning effect)
    while (distance > (MAX_CAM_DIST*MAX_CAM_DIST)) {
        fc->cam_xpos += ((8*((long)FAT_Cos16384(fc->cam_orientation))) >> 14);
        fc->cam_ypos += ((8*((long)FAT_Sin16384(fc->cam_orientation))) >> 14);

        dx = sprite->xpos-fc->cam_xpos;
        dy = sprite->ypos-fc->cam_ypos;
        distance = dx*dx + dy*dy;
    }

    // If we placed the camera in an invalid position, reset it
    if (!ValidCamMove(fc, fc->cam_xpos, fc->cam_ypos))  {ResetCam(fc, sprite, 0); return;}

// (3) If view of sprite is obstructed, pan camera a bit to resolve it
//     This step is now performed post-render instead of post-movement

#ifdef FORCE_CAMERA_BEHIND
    ForceCamBehind(fc, sprite);
#else
    // finally, point camera toward sprite
    fc->cam_orientation = FAT_GetArcTanYX(sprite->ypos-fc->cam_ypos, sprite->xpos-fc->cam_xpos);
#endif
}

//=============================================================================
// If view of sprite is obstructed, pan camera bit to resolve it
//=============================================================================
void PostRenderAdjustCam(FATCONFIG* fc, FATSPRITE* sprite) {
    short dx = sprite->xpos-fc->cam_xpos;
    short dy = sprite->ypos-fc->cam_ypos;
    long distance = dx*dx + dy*dy;
    short x0 = SCREEN_WIDTH_HALF - (sprite->height/4);  // would be /2, but we don't use the leftmost 1/4
    short viewblocked = 0;

    if (sprite->height == -1) return;

    // the +1 creates a bit of tolerance after all of the rouding in the engine
    if (((long)(FAT_GetDepthBuffer()->distance[x0+1]) * (long)(FAT_GetDepthBuffer()->distance[x0+1])) < distance+1)                                    viewblocked |= 0x01;
    if (((long)(FAT_GetDepthBuffer()->distance[x0+sprite->height/2-2]) * (long)(FAT_GetDepthBuffer()->distance[x0+sprite->height/2+2])) < distance+1)  viewblocked |= 0x02;

    if (viewblocked >= 3) {                                                    // view of both sides blocked -- cam positioning has failed
        PADSTATES key;
        if ((key = FAT_GetPadState()))                                         // if moving backwards
           if (key & PADSTATE_DOWN)  {ResetCam(fc, sprite, 1); return;}
        ResetCam(fc, sprite, 0);
    }
    else if (viewblocked==1)  RotateCam(fc, sprite, -ANGLE_015/4, distance);   // view of left side blocked, rotate cam counterclockwise
    else if (viewblocked==2)  RotateCam(fc, sprite,  ANGLE_015/4, distance);   // view of right side blocked, rotate cam clockwise
}


//=============================================================================
// as usual ... our main function
//=============================================================================
void _main(void) {
    PADSTATES       key    = 0;
    unsigned char*  bg_img = NULL;
    short           i,j;
    unsigned short* dest;
    TEXCONFIG       textures[2];
    TEXCONFIG       sprite_tex[8]; // eight sprites textures (one for each direction)
    FATSPRITE       sprite;        // one FAT sprite
    LCD_BUFFER      lcd;
    HANDLE          texfile = 0;
    unsigned char*  src     = NULL;
    short           result;
    COUNTER         frames_display;

    FATCONFIG fc = {
        MAP_WIDTH,  // width of map
        MAP_HEIGHT, // height of map
        map,        // pointer to map data
        textures,   // pointer to textures array
        1,          // number of sprites
        &sprite,    // the sprite array
        0,          // start pos_x -- this will be set below before first rendering
        0,          // start pos_y -- this will be set below before first rendering
        ANGLE_EAST, // orientation
        NULL,       // destination plane0
        NULL,       // destination plane1
        NULL,       // background image
        0           // frame counter
    };

    //-------------------------------------------------------------------------
    // before we try to do anything else we have to initialize the engine
    // its only valid to proceed if FAT_LOADOKAY gets returned
    //-------------------------------------------------------------------------
    if ((result = FAT_LoadEngine()) != FAT_LOADOKAY) {
        switch(result) {
            case FAT_NOTGHOSTSPACE: ST_helpMsg("ERROR: client no exepacked!");break;
            case FAT_NOTFOUND:      ST_helpMsg("ERROR: no matching engine found");break;
            case FAT_LOCKFAILED:    ST_helpMsg("ERROR: locking engine failed");break;
            case FAT_OUTOFMEM:      ST_helpMsg("ERROR: out of memory");break;
            default:                ST_helpMsg("ERROR: unknown errorcode");break;
        }
        return;
    }

    //-------------------------------------------------------------------------
    // try to load the used textures from fatdtx ...
    //-------------------------------------------------------------------------
    if ((texfile = FAT_GetHandleOfFile("fatdtx"))) {
        if ((src = FAT_LockHandleOfFile(texfile))) {
            // load house wall textures
            if (!FAT_LoadTextures(textures,src+2,TEXIDX_HOUSE1,2)) {
                src = 0; // just to indicate something went wrong
            }
            // load texture for doom guy
            if (src) {
                if (!FAT_LoadTextures(sprite_tex,src+2,TEXIDX_SOLDIER1,8)) {
                    src = 0; // just to indicate something went wrong
                }
            }
        }
    }

    //-------------------------------------------------------------------------
    // check if loading failed ...
    //-------------------------------------------------------------------------
    if (!texfile || !src) {
        FAT_UnloadEngine();
        if (texfile) FAT_UnlockHandleOfFile(texfile);
        ST_helpMsg("ERROR: cannot textures from fatdtx");
        return;
    }

    LCD_save(lcd);

    // initialize sprite structure
    // will set up its fields below
    FAT_InitSprite(&sprite);
    sprite.drawmode    = SPRITEMODE_SIZE64;
    sprite.nr_textures = 8;
    sprite.textures    = sprite_tex;
    sprite.orientation = 0;
    sprite.xpos        = 96+64;
    sprite.ypos        = 96+64;


    //-------------------------------------------------------------------------
    // allocate space for a background image and fill the lower part of the
    // light plane with 0xff to simulate a floor ...
    //-------------------------------------------------------------------------
    if (!(bg_img = (unsigned char*)malloc(1152*2))) {
        FAT_UnloadEngine();
        FAT_UnlockHandleOfFile(texfile);
        ST_helpMsg("ERROR: out of mem");
        return;
    }
    memset(bg_img,0x00,1152*2);

    dest = (unsigned short*)bg_img;
    for (i=0;i<6;i++) {
        for (j=0;j<48;j++) *dest++ = 0x0000;
        for (j=0;j<48;j++) *dest++ = 0xffff;
    }

    fc.background = (unsigned short*)bg_img;

    InstallDummyHandlerForINTs();

    //-------------------------------------------------------------------------
    // initialize grayscales, clear both planes and set the destination plane
    // addresses for the FAT-Engine
    //-------------------------------------------------------------------------
    if (!GrayOn()) {
        free(bg_img);
        RestoreHandlerForINTs();
        FAT_UnloadEngine();
        FAT_UnlockHandleOfFile(texfile);
        ST_helpMsg("ERROR: out of mem");
        return;
    }

    //-------------------------------------------------------------------------
    // display the FAT-Engine's splashscreen
    //-------------------------------------------------------------------------
    FAT_DisplayLogo(100);

    fc.dest_plane0 = GetPlane(0)+30*2;  // center vertically
    fc.dest_plane1 = GetPlane(1)+30*2;  // center vertically

    //-------------------------------------------------------------------------
    // setup calculator type specials
    //-------------------------------------------------------------------------
    if (TI89) {
        display_offsetx = 0;
        display_offsety = 0;
    }
    else {
        // TI92p or V200
        display_offsetx  = 40;
        display_offsety  = 14;
        fc.dest_plane0  += (14*30+4)/2;
        fc.dest_plane1  += (14*30+4)/2;
    }

    //-------------------------------------------------------------------------
    // initialize the GUI
    //-------------------------------------------------------------------------
    InitGUI(&frames_display);


    // some final sprite initialization here
    sprite.xpos = MAP_WIDTH*64/2;
    sprite.ypos = MAP_HEIGHT*64/2;
    sprite.orientation = ANGLE_000;
    sprite.angle = ANGLE_000;
    fc.cam_xpos = sprite.xpos + ((MAX_CAM_DIST*((long)FAT_Cos16384(ANGLE_180))) >> 14);
    fc.cam_ypos = sprite.ypos + ((MAX_CAM_DIST*((long)FAT_Sin16384(ANGLE_180))) >> 14);


    //-------------------------------------------------------------------------
    // the main rendering loop ...
    //-------------------------------------------------------------------------
    do {
        FAT_Render(&fc);                      // render image
        PostRenderAdjustCam(&fc, &sprite);    // some cam ajustment may be required

        frame_drawn(&fc,&frames_display); // handles frame counter drawing

        if ((key = FAT_GetPadState())) {
           if (key & PADSTATE_ESC)    break;
           if (key & PADSTATE_UPDOWN)    SpriteMove(&fc,&sprite,key & PADSTATE_UP);
           if (key & PADSTATE_LEFTRIGHT) SpriteTurn(&fc,&sprite,key & PADSTATE_RIGHT);
        }
    }
    while(1);

    //-------------------------------------------------------------------------
    // some clean stuff ...
    //-------------------------------------------------------------------------
    FAT_AllKeysReleased();
    GrayOff();
    free(bg_img);
    LCD_restore(lcd);
    ST_helpMsg(FAT_GetPoweredByString());
    FAT_UnlockHandleOfFile(texfile);  // unlock texture file
    FAT_UnloadEngine();
    RestoreHandlerForINTs();
    OSqclear(kbd_queue());
}

//#############################################################################
// Revision History
//#############################################################################
//
// $Log: fatd8.c,v $
// Revision 1.8  2002/05/08 15:35:14  tnussb
// (1) display of FAT splashscreen (FAT_DisplayLogo) added
// (2) GUI of demos completely revised
//
// Revision 1.7  2002/05/07 15:22:13  tnussb
// stupid double call of FAT_LoadEngine() fixed
//
// Revision 1.6  2002/04/30 17:41:45  tnussb
// using now defines from automatically generated headerfile indices.h for indices into
// our texture file
//
// Revision 1.5  2002/04/26 13:10:49  tnussb
// if FAT_LoadEngine() fails different strings will be displayed yet
//
// Revision 1.4  2002/04/25 19:35:34  tnussb
// generic commit for engine version v1.05
//
// Revision 1.3  2002/04/24 17:42:51  tnussb
// (1) using now FAT_GetPadState() to query key presses
//
// Revision 1.2  2002/04/22 07:30:37  tnussb
// very much improved third person view (Great work, Scott!)
//
// Revision 1.1  2002/04/18 12:13:38  tnussb
// initial version
//
