/******************************************************************************
*
* project name:   FAT-Engine
* file name:      fatd9.c
* initial date:   25/02/2001
* author:         thomas.nussbaumer@gmx.net
*
* description:    FAT-Engine - Demo 9 (Animated Textures Demo)
*
* $Id: fatd9.c,v 1.6 2002/05/08 15:35:14 tnussb Exp $
*
******************************************************************************/

#define USE_TI89
#define USE_TI92P
#define USE_V200

#define FAT_USEDEBUG    // used to export own variables to console

#include "../includes/fat.h"


#define  MAP_WIDTH        16
#define  MAP_HEIGHT       16

//-------------------------------------------------------------------
// defines the used map: 0 = no wall, 1 = texture1, 2 = texture2 etc.
// NOTE: the array tex64 which gets handed over to the engine as
//       part of the FATCONFIG structure has to as much textures as
//       you have specified in this map.
//       For example: if the highest number in the map is 3 then
//       tex64 has to hold 3 textures (64x64 pixels with 2 planes)
//-------------------------------------------------------------------

char map[MAP_WIDTH*MAP_HEIGHT] = {
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
     1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1,
     1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1,
     1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1,
     1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1,
     1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 0, 0, 1,
     1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
     1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1,
     1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1,
     1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1,
     1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1,
     1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1,
     1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1,
     1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
};


//-----------------------------------------------------------------------------
// include the utility routines ...
//-----------------------------------------------------------------------------
#include "utils.c"

//-----------------------------------------------------------------------------
// include file containing the texture and genericdata indices into fatdtx
// (automatically generated by TexMaker)
//-----------------------------------------------------------------------------
#include "indices.h"


short display_offsetx = 0; // will be set depending on calctype (TI89 or TI92p)
short display_offsety = 0; // will be set depending on calctype (TI89 or TI92p)


//=============================================================================
// initializes GUI (darkgray background, tiny logo and counter)
//=============================================================================
void InitGUI(COUNTER *counter) {
    static unsigned long empty[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

    memset(GetPlane(LIGHT_PLANE),0xff,LCD_SIZE);
    memset(GetPlane(DARK_PLANE),0xff,LCD_SIZE);

    Sprite32(158-32+display_offsetx,display_offsety+2,16,empty,GetPlane(0),SPRT_AND);
    Sprite32(158-32+display_offsetx,display_offsety+2,16,FAT_GetTinyLogo(),GetPlane(0),SPRT_OR);
    Sprite32(158-32+display_offsetx,display_offsety+2,16,empty,GetPlane(1),SPRT_AND);
    Sprite32(158-32+display_offsetx,display_offsety+2,16,FAT_GetTinyLogo()+16,GetPlane(1),SPRT_OR);

    InitCounter(counter,158-6*5+display_offsetx,99-10+display_offsety,5,0);
    DrawCounter(counter);
}


//=============================================================================
// output frame count ever 10th frame
//=============================================================================
void frame_drawn(FATCONFIG* fc,COUNTER* counter) {
    short i;
    if (fc->frame_counter%10==0) {
        // we starting incrementing at 2nd position and not on first!
        for (i=counter->length-2;i>=0;i--) {

            counter->newcount[i] = counter->newcount[i]+1;
            if (counter->newcount[i] < 10) break;
            else                           counter->newcount[i]=0;
        }
        DrawCounter(counter);
    }
}


//=============================================================================
// as usual ... our main function
//=============================================================================
void _main(void) {
    PADSTATES       key    = 0;
    unsigned char*  bg_img = NULL;
    TEXCONFIG       textures[7]; // texture[0] is the active texture, 1-6 are used for the animation
    TEXANIMATION    texanim;
    LCD_BUFFER      lcd;
    HANDLE          texfile = 0;
    unsigned char*  src     = NULL;
    short           result;
    COUNTER         frames_display;

    FATCONFIG fc = {
        MAP_WIDTH,  // width of map
        MAP_HEIGHT, // height of map
        map,        // pointer to map data
        textures,   // pointer to textures array
        0,          // number of sprites
        NULL,       // the sprite array
        96,         // start pos_x
        96,         // start pos_y
        ANGLE_EAST, // orientation
        NULL,       // destination plane0
        NULL,       // destination plane1
        NULL,       // background image
        0           // frame counter
    };

    //-------------------------------------------------------------------------
    // before we try to do anything else we have to initialize the engine
    // its only valid to proceed if FAT_LOADOKAY gets returned
    //-------------------------------------------------------------------------
    if ((result = FAT_LoadEngine()) != FAT_LOADOKAY) {
        switch(result) {
            case FAT_NOTGHOSTSPACE: ST_helpMsg("ERROR: client no exepacked!");break;
            case FAT_NOTFOUND:      ST_helpMsg("ERROR: no matching engine found");break;
            case FAT_LOCKFAILED:    ST_helpMsg("ERROR: locking engine failed");break;
            case FAT_OUTOFMEM:      ST_helpMsg("ERROR: out of memory");break;
            default:                ST_helpMsg("ERROR: unknown errorcode");break;
        }
        return;
    }

    //-------------------------------------------------------------------------
    // try to load the used textures from fatdtx ...
    //-------------------------------------------------------------------------
    if ((texfile = FAT_GetHandleOfFile("fatdtx"))) {
        if ((src = FAT_LockHandleOfFile(texfile))) {
            // load spinning cube textures
            if (!FAT_LoadTextures(&textures[1],src+2,TEXIDX_TICTCUBE1,6)) {
                src = 0; // just to indicate something went wrong
            }
        }
    }

    //-------------------------------------------------------------------------
    // check if loading failed ...
    //-------------------------------------------------------------------------
    if (!texfile || !src) {
        FAT_UnloadEngine();
        if (texfile) FAT_UnlockHandleOfFile(texfile);
        ST_helpMsg("ERROR: cannot textures from fatdtx");
        return;
    }

    LCD_save(lcd);

    texanim.dest       = &textures[0];
    texanim.sources    = &textures[1];
    texanim.anim_steps = 6;
    texanim.anim_idx   = 0;

    //-------------------------------------------------------------------------
    // allocate space for a background image and fill the lower part of the
    // light plane with 0xff to simulate a floor ...
    //-------------------------------------------------------------------------
    if (!(bg_img = (unsigned char*)malloc(1152*2))) {
        FAT_UnloadEngine();
        FAT_UnlockHandleOfFile(texfile);
        ST_helpMsg("ERROR: out of mem");
        return;
    }
    memset(bg_img,0xff,1152*2);
    fc.background = (unsigned short*)bg_img;

    InstallDummyHandlerForINTs();

    //-------------------------------------------------------------------------
    // initialize grayscales, clear both planes and set the destination plane
    // addresses for the FAT-Engine
    //-------------------------------------------------------------------------
    if (!GrayOn()) {
        free(bg_img);
        RestoreHandlerForINTs();
        FAT_UnloadEngine();
        FAT_UnlockHandleOfFile(texfile);
        ST_helpMsg("ERROR: out of mem");
        return;
    }

    //-------------------------------------------------------------------------
    // display the FAT-Engine's splashscreen
    //-------------------------------------------------------------------------
    FAT_DisplayLogo(100);

    fc.dest_plane0 = GetPlane(0)+30*2; // center vertically
    fc.dest_plane1 = GetPlane(1)+30*2; // center vertically

    //-------------------------------------------------------------------------
    // setup calculator type specials
    //-------------------------------------------------------------------------
    if (TI89) {
        display_offsetx = 0;
        display_offsety = 0;
    }
    else {
        // TI92p or V200
        display_offsetx  = 40;
        display_offsety  = 14;
        fc.dest_plane0  += (14*30+4)/2;
        fc.dest_plane1  += (14*30+4)/2;
    }

    //-------------------------------------------------------------------------
    // initialize the GUI
    //-------------------------------------------------------------------------
    InitGUI(&frames_display);

    //-------------------------------------------------------------------------
    // register handle of the texture file as debug variable for the FAT
    // console (use FATLIB_C.DLL to view them)
    //
    // (produces only code if FAT_USEDEBUG is set)
    //-------------------------------------------------------------------------
    FAT_DebugUseSlot(0,"texfile",&texfile,FAT_DBG_USHORT_DEC);

    //-------------------------------------------------------------------------
    // the main rendering loop ...
    //-------------------------------------------------------------------------
    do {
        //---------------------------------------------------------------------
        // FAT_UpdateAnimation ... animates the texture[0] by copying other
        // TEXCONFIG setups into it
        //---------------------------------------------------------------------
        FAT_UpdateAnimation(&texanim);

        FAT_Render(&fc);   // render image
        frame_drawn(&fc,&frames_display);

        if ((key = FAT_GetPadState())) {
            if (key & PADSTATE_ESC)    break;
            if (key & PADSTATE_UPDOWN) PlayerMove(&fc,key & PADSTATE_UP,0);

            if (key & PADSTATE_LEFTRIGHT) {
                // STRAFE KEY: 2ND (TI-89) / F1 (TI-92p)
                if (key & PADSTATE_KA) {
                    PlayerMove(&fc,key&PADSTATE_RIGHT,1);
                }
                else {
                    PlayerTurn(&fc,key&PADSTATE_RIGHT,ANGLE_015/2);
                }
            }
        }
    }
    while(1);

    //-------------------------------------------------------------------------
    // some clean stuff ...
    //-------------------------------------------------------------------------
    FAT_AllKeysReleased();
    GrayOff();
    free(bg_img);
    LCD_restore(lcd);
    ST_helpMsg(FAT_GetPoweredByString());
    FAT_UnlockHandleOfFile(texfile);  // unlock texture file
    FAT_UnloadEngine();
    RestoreHandlerForINTs();
    OSqclear(kbd_queue());
}

//#############################################################################
// Revision History
//#############################################################################
//
// $Log: fatd9.c,v $
// Revision 1.6  2002/05/08 15:35:14  tnussb
// (1) display of FAT splashscreen (FAT_DisplayLogo) added
// (2) GUI of demos completely revised
//
// Revision 1.5  2002/05/07 15:22:13  tnussb
// stupid double call of FAT_LoadEngine() fixed
//
// Revision 1.4  2002/05/03 11:30:16  tnussb
// using now FAT_UpdateAnimation()
//
// Revision 1.3  2002/04/30 17:41:45  tnussb
// using now defines from automatically generated headerfile indices.h for indices into
// our texture file
//
// Revision 1.2  2002/04/26 13:10:49  tnussb
// if FAT_LoadEngine() fails different strings will be displayed yet
//
// Revision 1.1  2002/04/25 19:38:43  tnussb
// initial version
//
//
